package com.sun.jmx.remote.protocol.iiop;

import java.security.ProtectionDomain;
import java.io.SerializablePermission;
import java.security.Permissions;
import java.security.PrivilegedActionException;
import java.security.AccessController;
import java.security.PrivilegedExceptionAction;
import java.rmi.NoSuchObjectException;
import java.rmi.Remote;
import javax.rmi.PortableRemoteObject;
import java.util.Properties;
import java.rmi.RemoteException;
import org.omg.CORBA.ORB;
import org.omg.CORBA.BAD_OPERATION;
import org.omg.CORBA.portable.Delegate;
import javax.rmi.CORBA.Stub;
import java.security.AccessControlContext;
import com.sun.jmx.remote.internal.IIOPProxy;

public class IIOPProxyImpl implements IIOPProxy
{
    private static final AccessControlContext STUB_ACC;
    
    public IIOPProxyImpl() {
    }
    
    public boolean isStub(Object o) {
        return o instanceof Stub;
    }
    
    public Object getDelegate(Object o) {
        return ((Stub)o)._get_delegate();
    }
    
    public void setDelegate(Object o, Object o2) {
        ((Stub)o)._set_delegate((Delegate)o2);
    }
    
    public Object getOrb(Object o) {
        try {
            return ((Stub)o)._orb();
        }
        catch (BAD_OPERATION cause) {
            throw new UnsupportedOperationException(cause);
        }
    }
    
    public void connect(Object o, Object o2) throws RemoteException {
        ((Stub)o).connect((ORB)o2);
    }
    
    public boolean isOrb(Object o) {
        return o instanceof ORB;
    }
    
    public Object createOrb(String[] array, Properties properties) {
        return ORB.init(array, properties);
    }
    
    public Object stringToObject(Object o, String s) {
        return ((ORB)o).string_to_object(s);
    }
    
    public String objectToString(Object o, Object o2) {
        return ((ORB)o).object_to_string((org.omg.CORBA.Object)o2);
    }
    
    public <T> T narrow(Object o, Class<T> clazz) {
        return PortableRemoteObject.narrow(o, clazz);
    }
    
    public void exportObject(Remote remote) throws RemoteException {
        PortableRemoteObject.exportObject(remote);
    }
    
    public void unexportObject(Remote remote) throws NoSuchObjectException {
        PortableRemoteObject.unexportObject(remote);
    }
    
    public Remote toStub(final Remote remote) throws NoSuchObjectException {
        if (System.getSecurityManager() == null) {
            return PortableRemoteObject.toStub(remote);
        }
        try {
            return (Remote)AccessController.doPrivileged(new PrivilegedExceptionAction<Remote>() {
                @Override
                public Remote run() throws Exception {
                    return PortableRemoteObject.toStub(remote);
                }
            }, IIOPProxyImpl.STUB_ACC);
        }
        catch (PrivilegedActionException ex) {
            if ((ex.getException() instanceof NoSuchObjectException)) {
                throw (NoSuchObjectException)ex.getException();
            }
            throw new RuntimeException("Unexpected exception type", ex.getException());
        }
    }
    
    static {
        Permissions permissions = new Permissions();
        permissions.add(new SerializablePermission("enableSubclassImplementation"));
        STUB_ACC = new AccessControlContext(new ProtectionDomain[] { new ProtectionDomain(null, permissions) });
    }
}
